@@ -249,25 +249,25 @@ gdk_pixdata_deserialize (GdkPixdata   *pixdata,
 }
 
 static gboolean
-diff2_rgb (guint8 *ip)
+diff2_rgb (const guint8 *ip)
 {
   return ip[0] != ip[3] || ip[1] != ip[4] || ip[2] != ip[5];
 }
 
 static gboolean
-diff2_rgba (guint8 *ip)
+diff2_rgba (const guint8 *ip)
 {
   return ip[0] != ip[4] || ip[1] != ip[5] || ip[2] != ip[6] || ip[3] != ip[7];
 }
 
-static guint8*			/* dest buffer bound */
-rl_encode_rgbx (guint8 *bp,	/* dest buffer */
-		guint8 *ip,	/* image pointer */
-		guint8 *limit,	/* image upper bound */
-		guint   n_ch)
+static guint8 *                         /* dest buffer bound */
+rl_encode_rgbx (guint8       *bp,       /* dest buffer */
+                const guint8 *ip,       /* image pointer */
+                const guint8 *limit,    /* image upper bound */
+                guint         n_ch)
 {
-  gboolean (*diff2_pix) (guint8 *) = n_ch > 3 ? diff2_rgba : diff2_rgb;
-  guint8 *ilimit = limit - n_ch;
+  gboolean (*diff2_pix) (const guint8 *) = n_ch > 3 ? diff2_rgba : diff2_rgb;
+  const guint8 *ilimit = limit - n_ch;
 
   while (ip < limit)
     {
@@ -275,7 +275,7 @@ rl_encode_rgbx (guint8 *bp,	/* dest buffer */
 
       if (diff2_pix (ip))
 	{
-	  guint8 *s_ip = ip;
+	  const guint8 *s_ip = ip;
 	  guint l = 1;
 
 	  ip += n_ch;
@@ -341,6 +341,7 @@ gdk_pixdata_from_pixbuf (GdkPixdata      *pixdata,
 {
   gpointer free_me = NULL;
   guint height, rowstride, encoding, bpp, length;
+  const guint8 *pixels = NULL;
   guint8 *img_buffer;
 
   g_return_val_if_fail (pixdata != NULL, NULL);
@@ -377,14 +378,16 @@ gdk_pixdata_from_pixbuf (GdkPixdata      *pixdata,
 				buf, 0, 0);
 	}
       else
-	buf = (GdkPixbuf *)pixbuf;
+        buf = (GdkPixbuf *)pixbuf;
+
+      pixels = gdk_pixbuf_read_pixels (buf);
       pad = rowstride;
       pad = MAX (pad, 130 + n_bytes / 127);
       data = g_new (guint8, pad + n_bytes);
       free_me = data;
       img_buffer = data;
       img_buffer_end = rl_encode_rgbx (img_buffer,
-				       buf->pixels, buf->pixels + n_bytes,
+				       pixels, pixels + n_bytes,
 				       bpp);
       length = img_buffer_end - img_buffer;
       if (buf != pixbuf)
@@ -392,7 +395,7 @@ gdk_pixdata_from_pixbuf (GdkPixdata      *pixdata,
     }
   else
     {
-      img_buffer = pixbuf->pixels;
+      img_buffer = (guint8 *) gdk_pixbuf_read_pixels (pixbuf);
       length = rowstride * height;
     }
 
@@ -409,6 +412,11 @@ gdk_pixdata_from_pixbuf (GdkPixdata      *pixdata,
   return free_me;
 }
 
+/* From glib's gmem.c */
+#define SIZE_OVERFLOWS(a,b) (G_UNLIKELY ((b) > 0 && (a) > G_MAXSIZE / (b)))
+
+#define RLE_OVERRUN(offset) (rle_buffer_limit == NULL ? FALSE : rle_buffer + (offset) > rle_buffer_limit)
+
 /**
  * gdk_pixbuf_from_pixdata:
  * @pixdata: a #GdkPixdata to convert into a #GdkPixbuf.
@@ -444,8 +452,35 @@ gdk_pixbuf_from_pixdata (const GdkPixdata *pixdata,
 
   bpp = (pixdata->pixdata_type & GDK_PIXDATA_COLOR_TYPE_MASK) == GDK_PIXDATA_COLOR_TYPE_RGB ? 3 : 4;
   encoding = pixdata->pixdata_type & GDK_PIXDATA_ENCODING_MASK;
+
+  g_debug ("gdk_pixbuf_from_pixdata() called on:");
+  g_debug ("\tEncoding %s", encoding == GDK_PIXDATA_ENCODING_RAW ? "raw" : "rle");
+  g_debug ("\tDimensions: %d x %d", pixdata->width, pixdata->height);
+  g_debug ("\tRowstride: %d, Length: %d", pixdata->rowstride, pixdata->length);
+  g_debug ("\tCopy pixels == %s", copy_pixels ? "true" : "false");
+
   if (encoding == GDK_PIXDATA_ENCODING_RLE)
     copy_pixels = TRUE;
+
+  /* Sanity check the length and dimensions */
+  if (SIZE_OVERFLOWS (pixdata->height, pixdata->rowstride))
+    {
+      g_set_error_literal (error, GDK_PIXBUF_ERROR,
+                           GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                           _("Image pixel data corrupt"));
+      return NULL;
+    }
+
+  if (encoding == GDK_PIXDATA_ENCODING_RAW &&
+      pixdata->length >= 1 &&
+      pixdata->length < pixdata->height * pixdata->rowstride - GDK_PIXDATA_HEADER_LENGTH)
+    {
+      g_set_error_literal (error, GDK_PIXBUF_ERROR,
+                           GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                           _("Image pixel data corrupt"));
+      return NULL;
+    }
+
   if (copy_pixels)
     {
       data = g_try_malloc_n (pixdata->height, pixdata->rowstride);
@@ -464,13 +499,26 @@ gdk_pixbuf_from_pixdata (const GdkPixdata *pixdata,
   if (encoding == GDK_PIXDATA_ENCODING_RLE)
     {
       const guint8 *rle_buffer = pixdata->pixel_data;
+      guint8 *rle_buffer_limit = NULL;
       guint8 *image_buffer = data;
       guint8 *image_limit = data + pixdata->rowstride * pixdata->height;
       gboolean check_overrun = FALSE;
 
-      while (image_buffer < image_limit)
+      if (pixdata->length >= 1)
+        rle_buffer_limit = pixdata->pixel_data + pixdata->length - GDK_PIXDATA_HEADER_LENGTH;
+
+      while (image_buffer < image_limit &&
+             (rle_buffer_limit != NULL || rle_buffer > rle_buffer_limit))
 	{
-	  guint length = *(rle_buffer++);
+	  guint length;
+
+	  if (RLE_OVERRUN(1))
+	    {
+	      check_overrun = TRUE;
+	      break;
+	    }
+
+	  length = *(rle_buffer++);
 
 	  if (length & 128)
 	    {
@@ -478,6 +526,11 @@ gdk_pixbuf_from_pixdata (const GdkPixdata *pixdata,
 	      check_overrun = image_buffer + length * bpp > image_limit;
 	      if (check_overrun)
 		length = (image_limit - image_buffer) / bpp;
+	      if (RLE_OVERRUN(bpp < 4 ? 3 : 4))
+	        {
+	          check_overrun = TRUE;
+	          break;
+	        }
 	      if (bpp < 4)	/* RGB */
 		do
 		  {
@@ -492,6 +545,11 @@ gdk_pixbuf_from_pixdata (const GdkPixdata *pixdata,
 		    image_buffer += 4;
 		  }
 		while (--length);
+	      if (RLE_OVERRUN(bpp))
+	        {
+	          check_overrun = TRUE;
+	          break;
+		}
 	      rle_buffer += bpp;
 	    }
 	  else
@@ -500,6 +558,11 @@ gdk_pixbuf_from_pixdata (const GdkPixdata *pixdata,
 	      check_overrun = image_buffer + length > image_limit;
 	      if (check_overrun)
 		length = image_limit - image_buffer;
+	      if (RLE_OVERRUN(length))
+	        {
+	          check_overrun = TRUE;
+	          break;
+		}
 	      memcpy (image_buffer, rle_buffer, length);
 	      image_buffer += length;
 	      rle_buffer += length;
