@@ -607,7 +607,7 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
 			g_set_error_literal (error,
 					     GDK_PIXBUF_ERROR,
 					     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-					     _("Invalid XPM header"));
+					     _("Dimensions do not match data"));
 			goto out;
 		}
 
