@@ -115,6 +115,28 @@ test_comment(void)
   g_object_unref (ref);
 }
 
+static void
+test_at_size (void)
+{
+  GError *error = NULL;
+  GdkPixbuf *ref;
+
+  if (!format_supported ("jpeg") || !format_supported ("png"))
+    {
+      g_test_skip ("format not supported");
+      return;
+    }
+
+  ref = gdk_pixbuf_new_from_file (g_test_get_filename (G_TEST_DIST, "bug753605-atsize.jpg", NULL), &error);
+  g_assert_no_error (error);
+  g_object_unref (ref);
+
+  ref = gdk_pixbuf_new_from_file_at_size (g_test_get_filename (G_TEST_DIST, "bug753605-atsize.jpg", NULL),
+					  50, 50, &error);
+  g_assert_no_error (error);
+  g_object_unref (ref);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -124,6 +146,7 @@ main (int argc, char **argv)
   g_test_add_func ("/pixbuf/jpeg/type9_rotation_exif_tag", test_type9_rotation_exif_tag);
   g_test_add_func ("/pixbuf/jpeg/bug775218", test_bug_775218);
   g_test_add_func ("/pixbuf/jpeg/comment", test_comment);
+  g_test_add_func ("/pixbuf/jpeg/at_size", test_at_size);
 
   return g_test_run ();
 }
